require 'active_support/concern'
require 'active_support'

module Subscribable
  module RelationAssigner
    extend ActiveSupport::Concern

    module ClassMethods
      def subscribable
        has_many :subscribers, class_name: 'Subscribable::SubRelation', as: :subscription
        has_many :subscriptions, class_name: 'Subscribable::SubRelation', as: :subscriber
      end

      def subscribable?
        true
      end
    end

    def subscribers_from(*args)
      subscribers.where(subscriber_type: args)
    end

    def subscriptions_to(*args)
      subscriptions.where(subscription_type: args)
    end

    def subscribe_to!(actor)
      sub = subscriptions.new
      sub.subscription = actor
      sub.save
    end

    def delete_subscription_to!(actor)
      subscription = subscription_with(actor)
      subscription.delete
    end

    def subscribed_to?(actor)
      query = actor.subscribers.where(subscriber_id: id, subscriber_type: self.class.name)
      query.empty? ? false : true
    end

    private
    def subscription_with(actor)
      subscriptions.where(subscription_id: actor.id, subscription_type: actor.class.name).first
    end
  end
end
